<?php
namespace Potato\PerformanceMonitoring\Model\ResourceModel;

use Magento\Framework\Api;
use Potato\PerformanceMonitoring\Api as PerformanceMonitoringApi;
use Potato\PerformanceMonitoring\Model as PerformanceMonitoringModel;
use Magento\Framework\Api\SearchCriteriaBuilder;

/**
 * Class CpuRepository
 */
class CpuRepository implements PerformanceMonitoringApi\CpuRepositoryInterface
{
    /** @var PerformanceMonitoringModel\CpuFactory  */
    protected $cpuFactory;

    /** @var PerformanceMonitoringModel\CpuRegistry  */
    protected $cpuRegistry;

    /** @var PerformanceMonitoringApi\Data\CpuSearchResultsInterfaceFactory  */
    protected $searchResultsFactory;

    /** @var Api\ExtensibleDataObjectConverter  */
    protected $extensibleDataObjectConverter;

    /** @var Cpu  */
    protected $cpuResource;

    /** @var SearchCriteriaBuilder  */
    protected $searchBuilder;

    /**
     * @param PerformanceMonitoringModel\CpuFactory $cpuFactory
     * @param PerformanceMonitoringModel\CpuRegistry $cpuRegistry
     * @param PerformanceMonitoringApi\Data\CpuSearchResultsInterfaceFactory $searchResultsFactory
     * @param Api\ExtensibleDataObjectConverter $extensibleDataObjectConverter
     * @param Cpu $cpuResource
     */
    public function __construct(
        PerformanceMonitoringModel\CpuFactory $cpuFactory,
        PerformanceMonitoringModel\CpuRegistry $cpuRegistry,
        PerformanceMonitoringApi\Data\CpuSearchResultsInterfaceFactory $searchResultsFactory,
        Api\ExtensibleDataObjectConverter $extensibleDataObjectConverter,
        Cpu $cpuResource,
        SearchCriteriaBuilder $searchBuilder
    ) {
        $this->cpuFactory = $cpuFactory;
        $this->cpuRegistry = $cpuRegistry;
        $this->cpuResource = $cpuResource;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->extensibleDataObjectConverter = $extensibleDataObjectConverter;
        $this->searchBuilder = $searchBuilder;
    }

    /**
     * @param \Potato\PerformanceMonitoring\Api\Data\CpuInterface $cpu
     * @return Cpu
     */
    public function save(PerformanceMonitoringApi\Data\CpuInterface $cpu)
    {
        $cpuData = $this->extensibleDataObjectConverter->toNestedArray(
            $cpu,
            [],
            PerformanceMonitoringApi\Data\CpuInterface::class
        );

        $cpuModel = $this->cpuFactory->create();
        $cpuModel->addData($cpuData);
        $cpuModel->setId($cpu->getId());
        $this->cpuResource->save($cpuModel);
        $this->cpuRegistry->push($cpuModel);
        $savedObject = $this->get($cpuModel->getId());
        return $savedObject;
    }

    /**
     * @param int $cpuId
     * @return PerformanceMonitoringApi\Data\CpuInterface
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function get($cpuId)
    {
        $cpuModel = $this->cpuRegistry->retrieve($cpuId);
        return $cpuModel->getDataModel();
    }

    /**
     * @param PerformanceMonitoringApi\Data\CpuInterface $cpu
     * @return bool
     */
    public function delete(PerformanceMonitoringApi\Data\CpuInterface $cpu)
    {
        return $this->deleteById($cpu->getId());
    }

    /**
     * @param int $cpuId
     * @return bool
     * @throws \Exception
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function deleteById($cpuId)
    {
        $cpuModel = $this->cpuRegistry->retrieve($cpuId);
        $cpuModel->getResource()->delete($cpuModel);
        $this->cpuRegistry->remove($cpuId);
        return true;
    }

    /**
     * @param Api\SearchCriteriaInterface $searchCriteria
     * @return mixed
     */
    public function getList(Api\SearchCriteriaInterface $searchCriteria)
    {
        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($searchCriteria);
        $collection = $this->cpuFactory->create()->getCollection();
        foreach ($searchCriteria->getFilterGroups() as $filterGroup) {
            $fields = [];
            $conditions = [];
            foreach ($filterGroup->getFilters() as $filter) {
                $condition = $filter->getConditionType() ? $filter->getConditionType() : 'eq';
                $fields[] = $filter->getField();
                $conditions[] = [$condition => $filter->getValue()];

            }
            if ($fields) {
                $collection->addFieldToFilter($fields, $conditions);
            }
        }
        $searchResults->setTotalCount($collection->getSize());
        $sortOrders = $searchCriteria->getSortOrders();
        if ($sortOrders) {
            /** @var Api\SortOrder $sortOrder */
            foreach ($searchCriteria->getSortOrders() as $sortOrder) {
                $collection->addOrder(
                    $sortOrder->getField(),
                    ($sortOrder->getDirection() == Api\SortOrder::SORT_ASC) ? 'ASC' : 'DESC'
                );
            }
        }
        $collection->setCurPage($searchCriteria->getCurrentPage());
        $collection->setPageSize($searchCriteria->getPageSize());

        $cpus = [];
        foreach ($collection as $cpuModel) {
            $cpus[] = $cpuModel->getDataModel();
        }
        $searchResults->setItems($cpus);
        return $searchResults;
    }

    /**
     * @param string $date
     * @return $this
     */
    public function removeDataByDate($date)
    {
        $criteria = $this
            ->searchBuilder
            ->addFilter(
                'created_at',
                $date,
                'lteq'
            )
            ->create();
        $items = $this->getList($criteria)->getItems();
        foreach ($items as $item) {
            $this->delete($item);
        }
        return $this;
    }

    /**
     * Create new empty cpu model
     * @return PerformanceMonitoringApi\Data\CpuInterface
     */
    public function create()
    {
        return $this->cpuRegistry->create();
    }
}
