<?php
namespace Potato\PerformanceMonitoring\Model\ResourceModel;

use Magento\Framework\Api;
use Potato\PerformanceMonitoring\Api as PerformanceMonitoringApi;
use Potato\PerformanceMonitoring\Model as PerformanceMonitoringModel;
use Magento\Framework\Api\SearchCriteriaBuilder;

/**
 * Class TransactionSystemRepository
 */
class TransactionSystemRepository implements PerformanceMonitoringApi\TransactionSystemRepositoryInterface
{
    /** @var PerformanceMonitoringModel\TransactionSystemFactory  */
    protected $transactionSystemFactory;

    /** @var PerformanceMonitoringModel\TransactionSystemRegistry  */
    protected $transactionSystemRegistry;

    /** @var PerformanceMonitoringApi\Data\TransactionSystemSearchResultsInterfaceFactory  */
    protected $searchResultsFactory;

    /** @var Api\ExtensibleDataObjectConverter  */
    protected $extensibleDataObjectConverter;

    /** @var TransactionSystem  */
    protected $transactionSystemResource;

    /** @var SearchCriteriaBuilder  */
    protected $searchBuilder;

    /**
     * TransactionSystemRepository constructor.
     * @param PerformanceMonitoringModel\TransactionSystemFactory $transactionSystemFactory
     * @param PerformanceMonitoringModel\TransactionSystemRegistry $transactionSystemRegistry
     * @param PerformanceMonitoringApi\Data\TransactionSystemSearchResultsInterfaceFactory $searchResultsFactory
     * @param Api\ExtensibleDataObjectConverter $extensibleDataObjectConverter
     * @param TransactionSystem $transactionSystemResource
     * @param SearchCriteriaBuilder $searchBuilder
     */
    public function __construct(
        PerformanceMonitoringModel\TransactionSystemFactory $transactionSystemFactory,
        PerformanceMonitoringModel\TransactionSystemRegistry $transactionSystemRegistry,
        PerformanceMonitoringApi\Data\TransactionSystemSearchResultsInterfaceFactory $searchResultsFactory,
        Api\ExtensibleDataObjectConverter $extensibleDataObjectConverter,
        TransactionSystem $transactionSystemResource,
        SearchCriteriaBuilder $searchBuilder
    ) {
        $this->transactionSystemFactory = $transactionSystemFactory;
        $this->transactionSystemRegistry = $transactionSystemRegistry;
        $this->transactionSystemResource = $transactionSystemResource;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->extensibleDataObjectConverter = $extensibleDataObjectConverter;
        $this->searchBuilder = $searchBuilder;
    }

    /**
     * @param \Potato\PerformanceMonitoring\Api\Data\TransactionSystemInterface $transactionSystem
     * @return TransactionSystem
     */
    public function save(PerformanceMonitoringApi\Data\TransactionSystemInterface $transactionSystem)
    {
        $transactionSystemData = $this->extensibleDataObjectConverter->toNestedArray(
            $transactionSystem,
            [],
            PerformanceMonitoringApi\Data\TransactionSystemInterface::class
        );

        $transactionSystemModel = $this->transactionSystemFactory->create();
        $transactionSystemModel->addData($transactionSystemData);
        $transactionSystemModel->setProcesses($transactionSystem->getProcesses());
        $transactionSystemModel->setId($transactionSystem->getId());
        $this->transactionSystemResource->save($transactionSystemModel);
        $this->transactionSystemRegistry->push($transactionSystemModel);
        $savedObject = $this->get($transactionSystemModel->getId());
        return $savedObject;
    }

    /**
     * @param int $transactionSystemId
     * @return PerformanceMonitoringApi\Data\TransactionSystemInterface
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function get($transactionSystemId)
    {
        $transactionSystemModel = $this->transactionSystemRegistry->retrieve($transactionSystemId);
        return $transactionSystemModel->getDataModel();
    }

    /**
     * @param PerformanceMonitoringApi\Data\TransactionSystemInterface $transactionSystem
     * @return bool
     */
    public function delete(PerformanceMonitoringApi\Data\TransactionSystemInterface $transactionSystem)
    {
        return $this->deleteById($transactionSystem->getId());
    }

    /**
     * @param int $transactionSystemId
     * @return bool
     * @throws \Exception
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function deleteById($transactionSystemId)
    {
        $transactionSystemModel = $this->transactionSystemRegistry->retrieve($transactionSystemId);
        $transactionSystemModel->getResource()->delete($transactionSystemModel);
        $this->transactionSystemRegistry->remove($transactionSystemId);
        return true;
    }

    /**
     * @param Api\SearchCriteriaInterface $searchCriteria
     * @return mixed
     */
    public function getList(Api\SearchCriteriaInterface $searchCriteria)
    {
        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($searchCriteria);
        $collection = $this->transactionSystemFactory->create()->getCollection();
        foreach ($searchCriteria->getFilterGroups() as $filterGroup) {
            $fields = [];
            $conditions = [];
            foreach ($filterGroup->getFilters() as $filter) {
                $condition = $filter->getConditionType() ? $filter->getConditionType() : 'eq';
                $fields[] = $filter->getField();
                $conditions[] = [$condition => $filter->getValue()];

            }
            if ($fields) {
                $collection->addFieldToFilter($fields, $conditions);
            }
        }
        $searchResults->setTotalCount($collection->getSize());
        $sortOrders = $searchCriteria->getSortOrders();
        if ($sortOrders) {
            /** @var Api\SortOrder $sortOrder */
            foreach ($searchCriteria->getSortOrders() as $sortOrder) {
                $collection->addOrder(
                    $sortOrder->getField(),
                    ($sortOrder->getDirection() == Api\SortOrder::SORT_ASC) ? 'ASC' : 'DESC'
                );
            }
        }
        $collection->setCurPage($searchCriteria->getCurrentPage());
        $collection->setPageSize($searchCriteria->getPageSize());

        $transactionSystems = [];
        foreach ($collection as $transactionSystemModel) {
            $transactionSystems[] = $transactionSystemModel->getDataModel();
        }
        $searchResults->setItems($transactionSystems);
        return $searchResults;
    }

    /**
     * @param string $date
     * @return $this
     */
    public function removeDataByDate($date)
    {
        $criteria = $this
            ->searchBuilder
            ->addFilter(
                'created_at',
                $date,
                'lteq'
            )
            ->create();
        $items = $this->getList($criteria)->getItems();
        foreach ($items as $item) {
            $this->delete($item);
        }
        return $this;
    }

    /**
     * Create new empty transaction system model
     * @return PerformanceMonitoringApi\Data\TransactionSystemInterface
     */
    public function create()
    {
        return $this->transactionSystemRegistry->create();
    }
}
