<?php
namespace Potato\PerformanceMonitoring\Model\ResourceModel;

use Magento\Framework\Api;
use Potato\PerformanceMonitoring\Api as PerformanceMonitoringApi;
use Potato\PerformanceMonitoring\Model as PerformanceMonitoringModel;
use Magento\Framework\Api\SearchCriteriaBuilder;

/**
 * Class TransactionWebRepository
 */
class TransactionWebRepository implements PerformanceMonitoringApi\TransactionWebRepositoryInterface
{
    /** @var PerformanceMonitoringModel\TransactionWebFactory  */
    protected $transactionWebFactory;

    /** @var PerformanceMonitoringModel\TransactionWebRegistry  */
    protected $transactionWebRegistry;

    /** @var PerformanceMonitoringApi\Data\TransactionWebSearchResultsInterfaceFactory  */
    protected $searchResultsFactory;

    /** @var Api\ExtensibleDataObjectConverter  */
    protected $extensibleDataObjectConverter;

    /** @var TransactionWeb  */
    protected $transactionWebResource;

    /** @var SearchCriteriaBuilder  */
    protected $searchBuilder;
    
    /**
     * TransactionWebRepository constructor.
     * @param PerformanceMonitoringModel\TransactionWebFactory $transactionWebFactory
     * @param PerformanceMonitoringModel\TransactionWebRegistry $transactionWebRegistry
     * @param PerformanceMonitoringApi\Data\TransactionWebSearchResultsInterfaceFactory $searchResultsFactory
     * @param Api\ExtensibleDataObjectConverter $extensibleDataObjectConverter
     * @param TransactionWeb $transactionWebResource
     * @param SearchCriteriaBuilder $searchBuilder
     */
    public function __construct(
        PerformanceMonitoringModel\TransactionWebFactory $transactionWebFactory,
        PerformanceMonitoringModel\TransactionWebRegistry $transactionWebRegistry,
        PerformanceMonitoringApi\Data\TransactionWebSearchResultsInterfaceFactory $searchResultsFactory,
        Api\ExtensibleDataObjectConverter $extensibleDataObjectConverter,
        TransactionWeb $transactionWebResource,
        SearchCriteriaBuilder $searchBuilder
    ) {
        $this->transactionWebFactory = $transactionWebFactory;
        $this->transactionWebRegistry = $transactionWebRegistry;
        $this->transactionWebResource = $transactionWebResource;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->extensibleDataObjectConverter = $extensibleDataObjectConverter;
        $this->searchBuilder = $searchBuilder;
    }

    /**
     * @param \Potato\PerformanceMonitoring\Api\Data\TransactionWebInterface $transactionWeb
     * @return TransactionWeb
     */
    public function save(PerformanceMonitoringApi\Data\TransactionWebInterface $transactionWeb)
    {
        $transactionWebData = $this->extensibleDataObjectConverter->toNestedArray(
            $transactionWeb,
            [],
            PerformanceMonitoringApi\Data\TransactionWebInterface::class
        );

        $transactionWebModel = $this->transactionWebFactory->create();
        $transactionWebModel->addData($transactionWebData);
        $transactionWebModel->setId($transactionWeb->getId());
        $this->transactionWebResource->save($transactionWebModel);
        $this->transactionWebRegistry->push($transactionWebModel);
        $savedObject = $this->get($transactionWebModel->getId());
        return $savedObject;
    }

    /**
     * @param int $transactionWebId
     * @return PerformanceMonitoringApi\Data\TransactionWebInterface
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function get($transactionWebId)
    {
        $transactionWebModel = $this->transactionWebRegistry->retrieve($transactionWebId);
        return $transactionWebModel->getDataModel();
    }
    
    /**
     * @param Api\SearchCriteriaInterface $searchCriteria
     * @return mixed
     */
    public function getList(Api\SearchCriteriaInterface $searchCriteria)
    {
        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($searchCriteria);
        $collection = $this->transactionWebFactory->create()->getCollection();
        foreach ($searchCriteria->getFilterGroups() as $filterGroup) {
            $fields = [];
            $conditions = [];
            foreach ($filterGroup->getFilters() as $filter) {
                $condition = $filter->getConditionType() ? $filter->getConditionType() : 'eq';
                $fields[] = $filter->getField();
                $conditions[] = [$condition => $filter->getValue()];

            }
            if ($fields) {
                $collection->addFieldToFilter($fields, $conditions);
            }
        }
        $searchResults->setTotalCount($collection->getSize());
        $sortOrders = $searchCriteria->getSortOrders();
        if ($sortOrders) {
            /** @var Api\SortOrder $sortOrder */
            foreach ($searchCriteria->getSortOrders() as $sortOrder) {
                $collection->addOrder(
                    $sortOrder->getField(),
                    ($sortOrder->getDirection() == Api\SortOrder::SORT_ASC) ? 'ASC' : 'DESC'
                );
            }
        }
        $collection->setCurPage($searchCriteria->getCurrentPage());
        $collection->setPageSize($searchCriteria->getPageSize());

        $transactionWebs = [];
        foreach ($collection as $transactionWebModel) {
            $transactionWebs[] = $transactionWebModel->getDataModel();
        }
        $searchResults->setItems($transactionWebs);
        return $searchResults;
    }

    /**
     * @param PerformanceMonitoringApi\Data\TransactionWebInterface $transactionWeb
     * @return bool
     */
    public function delete(PerformanceMonitoringApi\Data\TransactionWebInterface $transactionWeb)
    {
        return $this->deleteById($transactionWeb->getId());
    }

    /**
     * @param int $transactionWebId
     * @return bool
     * @throws \Exception
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function deleteById($transactionWebId)
    {
        $transactionWebModel = $this->transactionWebRegistry->retrieve($transactionWebId);
        $transactionWebModel->getResource()->delete($transactionWebModel);
        $this->transactionWebRegistry->remove($transactionWebId);
        return true;
    }

    /**
     * @param string $date
     * @return $this
     */
    public function removeDataByDate($date)
    {
        $criteria = $this
            ->searchBuilder
            ->addFilter(
                'created_at',
                $date,
                'lteq'
            )
            ->create();
        $items = $this->getList($criteria)->getItems();
        foreach ($items as $item) {
            $this->delete($item);
        }
        return $this;
    }

    /**
     * Create new empty transaction web model
     * @return PerformanceMonitoringApi\Data\TransactionWebInterface
     */
    public function create()
    {
        return $this->transactionWebRegistry->create();
    }
}
