<?php

namespace Custom\ProductManagement\Console\Command;

use Custom\ProductManagement\Model\Logger;
use Custom\ProductManagement\Model\Status as ProductStatus;
use Magento\Framework\Exception\LocalizedException;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Input\InputArgument;
use Magento\Framework\App\Filesystem\DirectoryList;

/**
 * Custom ProductManagement product status abstract console command.
 *
 * @category Custom
 * @package  Custom_ProductManagement
 * @author   Piotr Pierzak <piotrek.pierzak@gmail.com>
 */
abstract class AbstractStatusCommand extends Command
{
    const ARGUMENT_ATTRIBUTE = 'attribute';
    const ARGUMENT_FILE = 'file';

    /**
     * @var ProductStatus
     */
    private $productStatus;

    /**
     * @var Logger
     */
    private $logger;

    /**
     * @var DirectoryList
     */
    private $directoryList;

    /**
     * @var InputInterface
     */
    private $input;

    /**
     * Object constructor.
     *
     * @param ProductStatus $productStatus
     * @param Logger $logger
     * @param DirectoryList $directoryList
     */
    public function __construct(
        ProductStatus $productStatus,
        Logger $logger,
        DirectoryList $directoryList
    ) {
        $this->productStatus = $productStatus;
        $this->logger = $logger;
        $this->directoryList = $directoryList;

        parent::__construct();
    }

    /**
     * @return int
     */
    abstract protected function getProductStatus();

    /**
     * @return array
     */
    protected function getCommandDefinition()
    {
        return [
            new InputArgument(
                self::ARGUMENT_ATTRIBUTE,
                InputArgument::REQUIRED,
                'Attribute'
            ),
            new InputArgument(
                self::ARGUMENT_FILE,
                InputArgument::REQUIRED,
                'File'
            ),
        ];
    }

    /**
     * @return array
     */
    protected function getAllowedAttributes()
    {
        return ['barcode', 'name'];
    }

    /**
     * @return string
     * @throws LocalizedException
     */
    protected function getAttribute()
    {
        $attribute = $this->input->getArgument(self::ARGUMENT_ATTRIBUTE);
        if (!in_array($attribute, $this->getAllowedAttributes(), true)) {
            throw new LocalizedException(__('Unhandled attribute provided.'));
        }

        return $attribute;
    }

    /**
     * @return string
     * @throws LocalizedException
     */
    protected function getFilePath()
    {
        $filePath = $this->input->getArgument(self::ARGUMENT_FILE);
        if (strpos($filePath, 0, 1) !== '/') {
            $filePath = $this->directoryList->getRoot() . '/' . $filePath;
        }

        if (!file_exists($filePath)) {
            throw new LocalizedException(__('File "%1" does not exists.', $filePath));
        }

        if (!is_readable($filePath)) {
            throw new LocalizedException(__('File "%1" is not readable.', $filePath));
        }

        return $filePath;
    }

    /**
     * {@inheritdoc}
     *
     * @return int
     * @throws \Exception
     */
    protected function execute(
        InputInterface $input,
        OutputInterface $output
    ) {
        $this->input = $input;

        try {
            $attribute = $this->getAttribute();

            $filePath = $this->getFilePath();
            $data = explode("\n", file_get_contents($filePath));
        } catch (LocalizedException $e) {
            $output->writeln('<error>' . $e->getMessage() . '</error>');

            return -1;
        }

        $this->logger->logInfo('Starting bulk product status update...');

        try {
            $this->productStatus
                ->setAttribute($attribute)
                ->setAttributeValues($data)
                ->setProductStatus($this->getProductStatus())
                ->process();

            $this->logger->logInfo('Bulk product status Update has been finished');
        } catch (\Exception $e) {
            $this->logger->logError('During bulk product status update process error has occurred.');
            $this->logger->logError('Details: ' . $e->getMessage());

            return -1;
        }

        return 0;
    }
}
