<?php

namespace Custom\SalesReport\Model;

use Magento\Framework\App\Area;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Mail\Template\TransportBuilder;
use Magento\Framework\Translate\Inline\StateInterface;
use Magento\Store\Model\Store;
use Psr\Log\LoggerInterface;

/**
 * Custom SalesReport email model.
 *
 * @category Custom
 * @package  Custom_SalesReport
 * @author   Piotr Pierzak <piotrek.pierzak@gmail.com>
 */
class Email
{
    const TEMPLATE_SALES_REPORT_DAiLY = 'salesreport_daily';

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * @var StateInterface
     */
    private $inlineTranslation;

    /**
     * @var TransportBuilder
     */
    private $transportBuilder;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * Object initialization.
     *
     * @param ScopeConfigInterface $scopeConfig
     * @param StateInterface       $inlineTranslation
     * @param TransportBuilder     $transportBuilder
     * @param LoggerInterface      $logger
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        StateInterface $inlineTranslation,
        TransportBuilder $transportBuilder,
        LoggerInterface $logger
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->inlineTranslation = $inlineTranslation;
        $this->transportBuilder = $transportBuilder;
        $this->logger = $logger;
    }

    /**
     * @param string   $template Email template path.
     * @param array    $sendTo Recipient information.
     * @param array    $templateParams Template params.
     * @param int|null $storeId Store id.
     *
     * @return Email
     */
    public function sendEmailTemplate(
        $template,
        $sendTo = [],
        $templateParams = [],
        $storeId = Store::DEFAULT_STORE_ID
    ) {
        $data = new \Magento\Framework\DataObject();
        $data->setData($templateParams);

        $this->inlineTranslation->suspend();

        try {
            $transport = $this->transportBuilder
                ->setTemplateIdentifier($template)
                ->setTemplateOptions(
                    [
                        'area' => Area::AREA_FRONTEND,
                        'store' => $storeId,
                    ]
                )
                ->setTemplateVars(['data' => $data])
                ->setFrom($this->getSender())
                ->addTo($sendTo)
                ->getTransport();

            $transport->sendMessage();
            $this->inlineTranslation->resume();
        } catch (\Exception $e) {
            $this->inlineTranslation->resume();
            $this->logger->error($e);
        }

        return $this;
    }

    /**
     * @param array $sendTo
     * @param array $templateParams
     *
     * @return Email
     */
    public function sendDailyReport($sendTo = [], $templateParams = [])
    {
        $this->sendEmailTemplate(
            self::TEMPLATE_SALES_REPORT_DAiLY,
            $sendTo,
            $templateParams
        );

        return $this;
    }

    /**
     * @return array
     */
    private function getSender()
    {
        $senderName = $this->scopeConfig->getValue(
            'trans_email/ident_general/name'
        );
        $senderEmail = $this->scopeConfig->getValue(
            'trans_email/ident_general/email'
        );

        return [
            'name' => $senderName,
            'email' => $senderEmail,
        ];
    }
}
