<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the mageplaza.com license that is
 * available through the world-wide-web at this URL:
 * https://mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    Mageplaza
 * @package     Mageplaza_FrequentlyBought
 * @copyright   Copyright (c) 2017 Mageplaza (https://www.mageplaza.com/)
 * @license     http://mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\FrequentlyBought\Block\Product\ProductList;

use Magento\Catalog\Api\ProductRepositoryInterface;

/**
 * Class FrequentlyBought
 * @package Mageplaza\FrequentlyBought\Block\Product\ProductList
 */
class FrequentlyBought extends \Magento\Catalog\Block\Product\ProductList\Related
{
    /**
     * @var \Magento\Framework\Pricing\Helper\Data
     */
    private $priceHelper;

    /**
     * @var \Mageplaza\FrequentlyBought\Helper\Data
     */
    private $fbtDataHelper;

    /**
     * @var \Magento\Catalog\Api\ProductRepositoryInterface
     */
    private $productRepository;

    /**
     * Constructor
     *
     * @param \Magento\Catalog\Block\Product\Context $context
     * @param \Magento\Checkout\Model\ResourceModel\Cart $checkoutCart
     * @param \\Magento\Catalog\Model\Product\Visibility $catalogProductVisibility
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Framework\Module\Manager $moduleManager
     * @param \Magento\Framework\Pricing\Helper\Data $priceHelper
     * @param ProductRepositoryInterface $productRepository
     * @param \Mageplaza\FrequentlyBought\Helper\Data $fbtDataHelper
     * @param array $data
     */
    public function __construct(
        \Magento\Catalog\Block\Product\Context $context,
        \Magento\Checkout\Model\ResourceModel\Cart $checkoutCart,
        \Magento\Catalog\Model\Product\Visibility $catalogProductVisibility,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Framework\Module\Manager $moduleManager,
        \Magento\Framework\Pricing\Helper\Data $priceHelper,
        ProductRepositoryInterface $productRepository,
        \Mageplaza\FrequentlyBought\Helper\Data $fbtDataHelper,
        array $data = []
    ) {
        parent::__construct(
            $context,
            $checkoutCart,
            $catalogProductVisibility,
            $checkoutSession,
            $moduleManager,
            $data
        );
        $this->priceHelper = $priceHelper;
        $this->registry = $context->getRegistry();
        $this->productRepository = $productRepository;
        $this->storeManager = $context->getStoreManager();
        $this->fbtDataHelper = $fbtDataHelper;
    }

    /**
     * Get heading label
     *
     * @return string
     */
    public function getTitleBlock()
    {
        return $this->fbtDataHelper->getConfig('block_name');
    }

    /**
     * Get price with currency
     *
     * @param   float $price
     * @return string
     */
    public function getPriceWithCurrency($price)
    {
        return $this->priceHelper->currency($price, true, false);
    }

    /**
     * Get price without currency
     *
     * @param   object $product
     * @return float
     */
    public function getPriceAmount($product)
    {
        if ($product->getTypeId() == 'grouped' || $product->getTypeId() == 'bundle') {
            return 0;
        }
        return $product->getPriceInfo()->getPrice('final_price')->getAmount()->getValue();
    }

    /**
     * Get product
     *
     * @return object
     */
    public function getCurrentProduct()
    {
        return $this->registry->registry('current_product');
    }

    /**
     * Get all custom option product
     *
     * @param int $productId
     * @return string
     */
    public function getCustomOption($productId = null)
    {
        $product = $this->getProductById($productId);
        $option = $this->getLayout()->getBlock('mageplaza.frequently.bought.product.info.options')
            ->setProduct($product)
            ->toHtml();
        return $option;
    }

    /**
     * Get option product
     *
     * @param int $productId
     * @return string
     */
    public function getOptionWrapper($productId = null)
    {
        $html = '';
        $product = $this->getProductById($productId);
        $productType = $product->getTypeId();
        switch ($productType) {
            case 'configurable' :
                $html = $this->getLayout()->createBlock('Mageplaza\FrequentlyBought\Block\Product\View\Type\Configurable');
                break;
            case 'grouped':
                $html = $this->getLayout()->createBlock('\Magento\GroupedProduct\Block\Product\View\Type\Grouped')
                    ->setTemplate('Mageplaza_FrequentlyBought::product/view/type/grouped.phtml');
                break;
            case 'bundle':
                $html = $this->getLayout()->getBlock('mageplaza.fbt.product.info.bundle.options');
                break;
            case 'downloadable':
                $html = $this->getLayout()->createBlock('\Magento\Downloadable\Block\Catalog\Product\Links')
                    ->setTemplate('Mageplaza_FrequentlyBought::product/view/type/downloadable/links.phtml');
                break;
        }
        if ($html) {
            return $html->setProduct($product)->toHtml();
        }
        return $html;
    }

    /**
     * Get product by id
     *
     * @param int $productId
     * @return object
     */
    private function getProductById($productId = null)
    {
        $storeId = $this->storeManager->getStore()->getId();
        if ($productId) {
            $product = $this->productRepository->getById($productId, false, $storeId);
        } else {
            $product = $this->getCurrentProduct();
        }
        return $product;
    }

    /**
     * Get separator image config
     *
     * @return string
     */
    public function getSeparatorImage()
    {
        return $this->fbtDataHelper->getIcon();
    }

    /**
     * Get product limit config
     *
     * @return int
     */
    public function getItemLimit()
    {
        return (int) $this->fbtDataHelper->getConfig('item_limit');
    }

    /**
     * Get add to wishlist config
     *
     * @return bop
     */
    public function getShowWishList()
    {
        return $this->fbtDataHelper->getConfig('enable_add_to_wishlist');
    }
}
