<?php
/**
 * Copyright © 2017 Mageside. All rights reserved.
 * See MS-LICENSE.txt for license details.
 */

namespace Mageside\BackupToDropbox\Model;

use Magento\Framework\App\Filesystem\DirectoryList;

/**
 * Class Dropbox
 * @package Mageside\BackupToDropbox\Model
 */
class Dropbox extends \Magento\Backup\Model\Backup
{
    /**
     * Default backup directory
     */
    const DEFAULT_BACKUP_DIRECTORY = 'backups';

    /**
     * Running file flag name
     */
    const RUNNING_FILE_NAME = 'running.flag';

    /**
     * @var array
     */
    protected $_messages;

    /**
     * @var Lib\Dropbox
     */
    protected $_dbx;

    /**
     * @var \Magento\Backup\Helper\Data
     */
    protected $_helper;

    /**
     * @var \Magento\Framework\Translate\Inline\StateInterface
     */
    protected $_inlineTranslation;

    /**
     * @var \Magento\Framework\Backup\Factory
     */
    protected $_backupFactory;

    /**
     * @var \Magento\Framework\App\MaintenanceMode
     */
    protected $_maintenanceMode;

    /**
     * @var \Mageside\BackupToDropbox\Helper\Backup
     */
    protected $_magesideHelper;

    /**
     * @var \Magento\Framework\App\ResourceConnection
     */
    protected $_connection;

    /**
     * @var \Magento\Framework\Mail\Template\TransportBuilder
     */
    protected $_mailer;

    /**
     * File
     *
     * @var \Magento\Framework\Filesystem\Driver\File
     */
    protected $_file;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var \Magento\Framework\Archive\Gz
     */
    protected $_archiveGz;

    /**
     * @var \Magento\Framework\Filesystem\Directory\ReadInterface
     */
    protected $_directoryRoot;

    /**
     * Dropbox constructor.
     * @param \Magento\Backup\Helper\Data $helper
     * @param \Magento\Framework\App\ResourceConnection $resource
     * @param \Magento\Framework\Locale\ResolverInterface $localeResolver
     * @param \Magento\Backend\Model\Auth\Session $authSession
     * @param \Magento\Framework\Encryption\EncryptorInterface $encryptor
     * @param \Magento\Framework\Filesystem $filesystem
     * @param \Magento\Framework\Backup\Factory $backupFactory
     * @param \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder
     * @param \Magento\Framework\Translate\Inline\StateInterface $inlineTranslation
     * @param \Mageside\BackupToDropbox\Helper\Backup $magesideHelper
     * @param \Magento\Framework\App\MaintenanceMode $maintenanceMode
     * @param \Magento\Framework\Filesystem\Driver\File $file
     * @param \Psr\Log\LoggerInterface $logger
     * @param Lib\Dropbox $dropbox
     * @param \Magento\Framework\Archive\Gz $archiveGz
     * @param array $data
     */
    public function __construct(
        \Magento\Backup\Helper\Data $helper,
        \Magento\Framework\App\ResourceConnection $resource,
        \Magento\Framework\Locale\ResolverInterface $localeResolver,
        \Magento\Backend\Model\Auth\Session $authSession,
        \Magento\Framework\Encryption\EncryptorInterface $encryptor,
        \Magento\Framework\Filesystem $filesystem,
        \Magento\Framework\Backup\Factory $backupFactory,
        \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder,
        \Magento\Framework\Translate\Inline\StateInterface $inlineTranslation,
        \Mageside\BackupToDropbox\Helper\Backup $magesideHelper,
        \Magento\Framework\App\MaintenanceMode $maintenanceMode,
        \Magento\Framework\Filesystem\Driver\File $file,
        \Psr\Log\LoggerInterface $logger,
        \Mageside\BackupToDropbox\Model\Lib\Dropbox $dropbox,
        \Magento\Framework\Archive\Gz $archiveGz,
        $data = []
    ) {
        $this->_connection = $resource;
        $this->_mailer = $transportBuilder;
        $this->_inlineTranslation = $inlineTranslation;
        $this->_magesideHelper = $magesideHelper;
        $this->_backupFactory = $backupFactory;
        $this->_helper = $helper;
        $this->_maintenanceMode = $maintenanceMode;
        $this->_messages = [];
        $this->_file = $file;
        $this->_directoryRoot = $filesystem->getDirectoryRead(DirectoryList::ROOT);
        $this->_archiveGz = $archiveGz;
        $this->_logger = $logger;
        $this->_dbx = $dropbox;

        parent::__construct($helper, $localeResolver, $authSession, $encryptor, $filesystem, $data);
    }

    /**
     * @return \Kunnu\Dropbox\Dropbox
     */
    public function getDBX()
    {
        return $this->_dbx;
    }

    /**
     * @return array
     */
    public function getMessages()
    {
        return $this->_messages;
    }

    /**
     * @return array|object
     */
    public function scheduledBackupToDropbox()
    {
        if (!$this->_magesideHelper->isBackupToDropboxEnabled()) {
            return $this;
        }

        if ($this->isRunning()) {
            return $this;
        }

        $dbxClient = $this->getDBX();

        /* Validate connection */
        if (!$this->checkConnection()) {
            return $this->_messages;
        }

        $this->setData("backupsDir", ($this->_helper->getBackupsDir()));
        $this->setData("rootDir", ($this->_directoryRoot->getAbsolutePath()));

        /* Create backup */
        set_time_limit(0);
        try {
            $type = $this->_magesideHelper->getConfigModule('XML_PATH_BACKUP_TYPE');

            $backupManager = $this->createBackupManager($type);
            $successMessage = $this->_helper->getCreateSuccessMessageByType($type);

            $this->setMaintenanceModeOption();
            $this->setRunningFlag(true);
            $backupManager->create();
            $this->setRunningFlag(false);
            $this->_messages[] = ['type' => 'success', 'message' => $successMessage];
            if ($type == \Magento\Framework\Backup\Factory::TYPE_DB) {
                $backupManager = $this->archivePack($backupManager);
            }
        } catch (\Magento\Framework\Backup\Exception\NotEnoughFreeSpace $e) {
            $this->unSetMaintenanceModeOption();
            $this->_messages[] = ['type' => 'error', 'message' => __('You need more free space to create a backup.')];
            $this->_logger->info($e->getMessage());
        } catch (\Magento\Framework\Backup\Exception\NotEnoughPermissions $e) {
            $this->unSetMaintenanceModeOption();
            $this->_logger->info($e->getMessage());
            $this->_messages[] = ['type' => 'error', 'message' => __('You need more permissions to create a backup.')];
        } catch (\Exception $e) {
            $this->unSetMaintenanceModeOption();
            $this->_messages[] = ['type' => 'error', 'message' => __('We can\'t create the backup right now.')];
            $this->_logger->info($e->getMessage());
        }
        $this->setRunningFlag();
        $this->unSetMaintenanceModeOption();

        /* Send backup to Dropbox */
        try {
            $this->uploadFiletoDropbox($dbxClient, $backupManager);
        } catch (\Exception $e) {
            $this->_messages[] = ['type' => 'error', 'message' => $e->getMessage()];
            return $this->_messages;
        }

        /* Delete old backups */
        $info = [];
        if ($this->_magesideHelper->getConfigModule('XML_PATH_BACKUP_DELETE_OLD')) {
            $days = $this->_magesideHelper->getConfigModule('XML_PATH_BACKUP_AFTER_DAYS');
            $info = $this->deleteOldBackup($dbxClient, $days);
        }

        /* Close connection */
        $this->_connection->closeConnection();

        if ($this->_magesideHelper->getConfigModule('XML_PATH_BACKUP_SEND_EMAIL')) {
            $this->sendMail(['delete_files' => $info, 'messages' => $this->_messages]);
        }
        return $this->_messages;
    }

    /**
     * @param bool $flag
     */
    public function setRunningFlag($flag = false)
    {
        if ($flag) {
            if (!file_exists($this->_helper->getBackupsDir() .DIRECTORY_SEPARATOR. self::RUNNING_FILE_NAME)) {
                $file = $this->_file
                    ->fileOpen(
                        $this->_helper->getBackupsDir() .DIRECTORY_SEPARATOR. self::RUNNING_FILE_NAME,
                        'w+'
                    );
                $this->_file->fileClose($file);
            }
        } else {
            if (file_exists($this->_helper->getBackupsDir() .DIRECTORY_SEPARATOR. self::RUNNING_FILE_NAME)) {
                $this->_file
                    ->deleteFile(
                        $this->_helper->getBackupsDir() .DIRECTORY_SEPARATOR. self::RUNNING_FILE_NAME
                    );
            }
        }
    }

    /**
     * Validate connection
     * @return bool
     */
    public function checkConnection()
    {
        $dbxClient = $this->getDBX();
        try {
            /** @var \Kunnu\Dropbox\Models\Account $accountInfo */
            $accountInfo = $dbxClient->getCurrentAccount();
            $_html = 'Connection to Dropbox success:<ul class="second-level">';
            $_html .= '<li>User Name: ' . $accountInfo->getDisplayName() . '</li>';
            $_html .= '<li>User Email: ' . $accountInfo->getEmail() . '</li></ul>';
            $this->_messages[] = [
                'type' => 'success',
                'message' => $_html
            ];
            return true;
        } catch (\Exception $e) {
            $this->_logger->info($e);
            $this->_messages[] = ['type' => 'error', 'message' => $e->getMessage()];
            return false;
        }
    }

    /**
     * @param \Kunnu\Dropbox\Dropbox $dbxClient
     * @param $fileInfo
     * @return $this
     */
    public function uploadFileToDropbox(\Kunnu\Dropbox\Dropbox $dbxClient, $fileInfo)
    {
        try {
            if (is_object($fileInfo)) {
                $fileName = $fileInfo->getBackupFilename();
                $filePath = $fileInfo->getBackupsDir() . DIRECTORY_SEPARATOR . $fileName;
                $type = $fileInfo->getType();
            } else {
                $fileName = $fileInfo['name'];
                $filePath = $fileInfo['full_path'];
                $type = $fileInfo['type'];
            }
            $filePathOnDropbox = '/backups/';
            $fileNameOnDropbox = $fileName;
            $destinationPath = $filePathOnDropbox . $fileNameOnDropbox;

            try {
                $dbxClient->upload($filePath, $destinationPath);
                $_html = 'Backup "' . $type . '" has been saved in the DropBox.';
                $_html .= '<ul class="second-level"><li>File name: ' . $fileNameOnDropbox . '</li></ul>';
                $this->_messages[] = ['type' => 'success', 'message' => $_html];
                $this->_file->deleteFile($filePath);
            } catch (\Exception $e) {
                $this->_logger->info($e->getMessage());
                $this->_messages[] = ['type' => 'error', 'message' => $e->getMessage()];
                $this->_file->deleteFile($filePath);
            }
        } catch (\Exception $e) {
            $this->_logger->info($e->getMessage());
            $this->_messages[] = ['type' => 'error', 'message' => $e->getMessage()];
        }
        return $this;
    }

    /**
     * @param \Kunnu\Dropbox\Dropbox $dbxClient
     * @param $days
     * @return array
     */
    public function deleteOldBackup(\Kunnu\Dropbox\Dropbox $dbxClient, $days)
    {
        $folderMataData = $dbxClient->listFolder('/backups');
        $deletedFiles = [];
        foreach ($folderMataData->getItems() as $file) {
            if (time() - strtotime($file->getClientModified()) >= $days * 24 * 60 * 60) {
                try {
                    $dbxClient->delete($file->getPathDisplay());
                    $deletedFiles[] = $file->getPathDisplay();
                } catch (\Exception $e) {
                    $this->_logger->info($e->getMessage());
                }
            }
        }
        return $deletedFiles;
    }

    /**
     * @param $content
     * @return $this
     */
    public function sendMail($content)
    {
        try {
            $this->_inlineTranslation->suspend();

            $transport = $this->_mailer
                ->setTemplateIdentifier('mageside_backup_to_dropbox_template')
                ->setTemplateOptions(
                    [
                        'area' => \Magento\Framework\App\Area::AREA_FRONTEND,
                        'store' => \Magento\Store\Model\Store::DEFAULT_STORE_ID,
                    ]
                )->setTemplateVars(['messages' => $content['messages'], 'delete_files' => $content['delete_files']])
                ->setFrom('general')
                ->addTo($this->_magesideHelper->getSender())
                ->getTransport();
            $transport->sendMessage();
            $this->_inlineTranslation->resume();
        } catch (\Exception $e) {
            $this->_logger->info($e->getMessage());
            $this->_messages[] = ['type' => 'error', 'message' => $e->getMessage()];
        }
        return $this;
    }

    /**
     * @param $backupManager
     * @return mixed
     */
    public function archivePack($backupManager)
    {
        $backUpFile = $backupManager->getBackupFilename();
        $backUpFolder = $backupManager->getBackupsDir();
        $backUpFilePatch = $backUpFolder.DIRECTORY_SEPARATOR.$backUpFile;
        if ($this->_magesideHelper->getConfigModule('XML_DB_BACKUP_FORMAT') == "gz") {
            $fileName = $backUpFile . '.gz';
            $destination = $backUpFolder . DIRECTORY_SEPARATOR . $fileName;
            try {
                $gzPacker = $this->_archiveGz;
                $gzPacker->pack($backUpFilePatch, $destination);
                $this->_file->deleteFile($backUpFilePatch);
                return ['full_path' => $destination, 'name' => $fileName, 'type' => 'db'];
            } catch (\Exception $e) {
                $this->_logger->info($e->getMessage());
                $this->_messages[] = ['type' => 'error', 'message' => __('We can\'t create archive.')];
                return $backupManager;
            }
        }
        return $backupManager;
    }

    /**
     * @return bool
     */
    public function setMaintenanceModeOption()
    {
        if ($this->_magesideHelper->getConfigModule('XML_PATH_BACKUP_MAINTENANCE_MODE')) {
            if (!$this->_maintenanceMode->set(true)) {
                $this->_messages[] = [
                    'type' => 'error',
                    'message' =>
                        __('You need more permissions to activate maintenance mode right now.')
                        . ' ' . __(
                            'To complete the rollback, please deselect '
                            . '"Put store into maintenance mode" or update your permissions.'
                        )
                ];
                return false;
            }
        }
        return true;
    }

    /**
     * @return $this
     */
    public function unSetMaintenanceModeOption()
    {
        $this->_maintenanceMode->set(false);
        return $this;
    }

    /**
     * @param $type
     * @return object \Magento\Framework\Backup\Factory
     */
    public function createBackupManager($type)
    {
        $backupManager = $this->_backupFactory->create(
            $type
        )->setBackupExtension(
            $this->_helper->getExtensionByType($type)
        )->setTime(
            time()
        )->setBackupsDir(
            $this->_helper->getBackupsDir()
        );
        if (!$this->_file->isExists($this->_helper->getBackupsDir())) {
            $this->_file->createDirectory($this->_helper->getBackupsDir());
        }
        if ($type != \Magento\Framework\Backup\Factory::TYPE_DB) {
            $backupManager
                ->setRootDir($this->getData('rootDir'))
                ->addIgnorePaths($this->_helper->getBackupIgnorePaths());
            $backupManager = ($this->_magesideHelper->getBackupIgnoreFolderPaths($backupManager));
        }
        return $backupManager;
    }

    /**
     * @return bool
     */
    public function isRunning()
    {
        if (file_exists($this->_helper->getBackupsDir() .DIRECTORY_SEPARATOR. self::RUNNING_FILE_NAME)) {
            return true;
        }
        return false;
    }
}
