<?php

namespace Potato\PerformanceMonitoring\Cron;

use Potato\PerformanceMonitoring\Model\Config;
use Potato\PerformanceMonitoring\Api\CpuRepositoryInterface;
use Potato\PerformanceMonitoring\Api\MemoryRepositoryInterface;
use Potato\PerformanceMonitoring\Api\TransactionSystemRepositoryInterface;
use Potato\PerformanceMonitoring\Api\TransactionWebRepositoryInterface;
use Potato\PerformanceMonitoring\Model\Mail\Sender;
use Psr\Log\LoggerInterface;
use Magento\Framework\App\CacheInterface;
use Magento\Framework\Stdlib\DateTime;

/**
 * Class TrackResource
 */
class TrackResource
{
    const LOCK_CACHE_INDEX = 'po_monitoring_lock';
    
    /** @var Config  */
    protected $config;

    /** @var TransactionWebRepositoryInterface  */
    protected $transactionWebRepository;

    /** @var TransactionSystemRepositoryInterface  */
    protected $transactionSystemRepository;

    /** @var CpuRepositoryInterface  */
    protected $cpuRepository;

    /** @var MemoryRepositoryInterface  */
    protected $memoryRepository;

    /** @var LoggerInterface  */
    protected $logger;

    /** @var CacheInterface  */
    protected $cache;
    
    /** @var Sender  */
    protected $sender;

    /**
     * TrackResource constructor.
     * @param Config $config
     * @param CpuRepositoryInterface $cpuRepository
     * @param MemoryRepositoryInterface $memoryRepository
     * @param TransactionSystemRepositoryInterface $transactionSystemRepository
     * @param TransactionWebRepositoryInterface $transactionWebRepository
     * @param LoggerInterface $logger
     * @param CacheInterface $cache
     * @param Sender $sender
     */
    public function __construct(
        Config $config,
        CpuRepositoryInterface $cpuRepository,
        MemoryRepositoryInterface $memoryRepository,
        TransactionSystemRepositoryInterface $transactionSystemRepository,
        TransactionWebRepositoryInterface $transactionWebRepository,
        LoggerInterface $logger,
        CacheInterface $cache,
        Sender $sender
    ) {
        $this->config = $config;
        $this->cpuRepository = $cpuRepository;
        $this->memoryRepository = $memoryRepository;
        $this->transactionSystemRepository = $transactionSystemRepository;
        $this->transactionWebRepository = $transactionWebRepository;
        $this->logger = $logger;
        $this->cache = $cache;
        $this->sender = $sender;
    }

    /**
     * @return $this
     */
    public function execute()
    {
        if (!$this->config->canUseModule()) {
            return $this;
        }
        try {
            $this
                ->trackLoadAvg()
                ->trackMemoryUsage()
                ->trackSystemTransaction()
            ;
        } catch (\Exception $e) {
            $this->logger->error($e->getMessage());
        }
        return $this;
    }

    /**
     * @return $this
     */
    protected function trackLoadAvg()
    {
        $load = sys_getloadavg();
        $currentDate = new \DateTime();
        $cpu = $this->cpuRepository->create();
        $cpu->setValue($load[0])
            ->setCreatedAt($currentDate->format(DateTime::DATETIME_PHP_FORMAT));
        $this->cpuRepository->save($cpu);
        return $this->sendAlerts($load[0]);
    }

    /**
     * Send alert email
     *
     * @param int $load
     * @return $this
     */
    protected function sendAlerts($load)
    {
        if (!$this->config->canUseAlert() ||
            $load < $this->config->getAlertCondition() ||
            $this->isLocked()
        ) {
            return $this;
        }
        
        try {
            $this->sender->sendAlert($load);
            $this->setLock();
        } catch (\Exception $e) {
            $this->logger->error($e->getMessage());
        }
        return $this;
    }

    /**
     * @return $this
     */
    protected function trackMemoryUsage()
    {
        $result = [];
        $status = [];
        exec('free -m 2>&1', $result, $status);
        if ($status !== 0) {
            return $this;
        }
        $memoryLine = [];
        $swapLine = [];
        foreach ($result as $line) {
            $values = explode(':', $line);
            if (!isset($values[0])) {
                continue;
            }
            switch ($values[0]) {
                case "Mem":
                    $memoryLine = array_values(
                        array_filter(
                            explode(" ", $line),
                            function ($value) {
                                return $value !== '';
                            }
                        )
                    );
                    break;
                case "Swap":
                    $swapLine = array_values(
                        array_filter(
                            explode(" ", $line),
                            function ($value) {
                                return $value !== '';
                            }
                        )
                    );
                    break;
            }
        }
        $currentDate = new \DateTime();
        $memoryValue = 0;
        if (array_key_exists(2, $memoryLine)) {
            $memoryValue = $memoryLine[2];
        }
        $swapUsage = 0;
        if (array_key_exists(2, $swapLine)) {
            $swapUsage = $swapLine[2];
        }
        $memory = $this->memoryRepository->create();
        $memory
            ->setValue($memoryValue)
            ->setSwap($swapUsage)
            ->setCreatedAt($currentDate->format(DateTime::DATETIME_PHP_FORMAT))
        ;
        $this->memoryRepository->save($memory);

        return $this;
    }

    /**
     * @return $this
     */
    protected function trackSystemTransaction()
    {
        $result = [];
        $status = [];
        exec('ps -o pid,%cpu,%mem,comm,cmd  --ppid 2 -N | sort -k 3 -nr', $result, $status);

        if ($status !== 0) {
            return $this;
        }

        $data = [];
        foreach($result as $_line) {
            $line = array_values(array_diff(explode(" ", $_line), ['']));
            if (!isset($line[1]) && !isset($line[2]) || ($line[1] == 0 && $line[2] == 0)) {
                continue;
            }
            $data[md5($line[3])][] = [
                'pid'       => $line[0],
                'cpu'       => $line[1],
                'memory'    => $line[2],
                'short_cmd' => $line[3],
                'cmd'       => $this->getCommand($line)
            ];
        }
        $currentDate = new \DateTime();
        foreach ($data as $group) {
            $groupedData = $this->mergeData($group);
            $transaction = $this->transactionSystemRepository->create();
            $transaction
                ->setCpu($groupedData['cpu'])
                ->setCmd($groupedData['cmd'])
                ->setMemory($groupedData['memory'])
                ->setProcesses($groupedData['processes'])
                ->setCreatedAt($currentDate->format(DateTime::DATETIME_PHP_FORMAT))
            ;
            $this->transactionSystemRepository->save($transaction);
        }
        return $this;
    }

    /**
     * @param $line
     * @return string
     */
    protected function getCommand($line)
    {
        unset($line[0]);
        unset($line[1]);
        unset($line[2]);
        unset($line[3]);
        return implode(' ', $line);
    }

    /**
     * @param array $group
     * @return array
     */
    protected function mergeData($group)
    {
        $result = [
            'cpu'       => 0,
            'memory'    => 0,
            'cmd'       => '',
            'processes' => []
        ];
        foreach ($group as $process) {
            $result['cpu'] += $process['cpu'];
            $result['memory'] += $process['memory'];
            $result['cmd'] = $process['short_cmd'];
            $result['processes'][] = [
                'pid'    => $process['pid'],
                'cpu'    => $process['cpu'],
                'memory' => $process['memory'],
                'cmd'    => $process['cmd']
            ];
        }
        return $result;
    }

    /**
     * @return bool
     */
    protected function isLocked()
    {
        if ($this->cache->load(self::LOCK_CACHE_INDEX)) {
            return true;
        }
        return false;
    }

    /**
     * @return $this
     */
    protected function setLock()
    {
        $this->cache->save(time(), self::LOCK_CACHE_INDEX, [],
            $this->config->getAlertFrequency()
        );
        return $this;
    }
}
