<?php

namespace Potato\PerformanceMonitoring\Model;

use Magento\Framework\App\RequestInterface;
use Potato\PerformanceMonitoring\Model\ResourceModel\TransactionSystem\CollectionFactory as TransactionSystemCollectionFactory;
use Potato\PerformanceMonitoring\Model\ResourceModel\TransactionWeb\CollectionFactory as TransactionWebCollectionFactory;
use Magento\Framework\Stdlib\DateTime;
use Magento\Framework\Stdlib\DateTime\TimezoneInterface;

/**
 * Class Chart
 */
class Chart
{
    //8 hours
    const TIME_LIMIT = 28800;

    /** @var RequestInterface  */
    protected $request;

    /** @var TransactionSystemCollectionFactory  */
    protected $transactionSystemCollectionFactory;

    /** @var TransactionWebCollectionFactory  */
    protected $transactionWebCollectionFactory;

    /** @var Config  */
    protected $config;
    
    /** @var TimezoneInterface  */
    protected $localeDate;
    
    protected $serializer = null;

    /**
     * Chart constructor.
     * @param RequestInterface $request
     * @param TransactionSystemCollectionFactory $transactionSystemCollectionFactory
     * @param TransactionWebCollectionFactory $transactionWebCollectionFactory
     * @param Config $config
     * @param TimezoneInterface $localeDate
     */
    public function __construct(
        RequestInterface $request,
        TransactionSystemCollectionFactory $transactionSystemCollectionFactory,
        TransactionWebCollectionFactory $transactionWebCollectionFactory,
        Config $config,
        TimezoneInterface $localeDate
    ) {
        $this->request = $request;
        $this->transactionSystemCollectionFactory = $transactionSystemCollectionFactory;
        $this->transactionWebCollectionFactory = $transactionWebCollectionFactory;
        $this->config = $config;
        $this->localeDate = $localeDate;
        if (class_exists('\Magento\Framework\Serialize\Serializer\Json')) {
            $this->serializer = \Magento\Framework\App\ObjectManager::getInstance()
                ->get('\Magento\Framework\Serialize\Serializer\Json');
        }
    }

    /**
     * Filtered Date From
     *
     * @return string
     */
    public function getDateFrom()
    {

        $from = $this->request->getParam('start_date', null);
        if (null === $from) {
            $currentDate = new \DateTime();
            $from = $currentDate->modify('-30 minutes')->format(DateTime::DATETIME_PHP_FORMAT);
        } else {
            $from = base64_decode($from);
        }
        return $from;
    }

    /**
     * Filtered Date To
     *
     * @return string
     */
    public function getDateTo()
    {
        $to = $this->request->getParam('end_date', null);
        if (null === $to) {
            $currentDate = new \DateTime();
            $to = $currentDate->format(DateTime::DATETIME_PHP_FORMAT);
        } else {
            $to = base64_decode($to);
        }
        return $to;
    }

    /**
     * @param bool $locale
     * @param string $date
     * @return \Zend_Json_Expr
     */
    public function formatDateToJs($date, $locale=false)
    {
        if ($locale) {
            $date = $this->convertDateToLocale($date);
        }
        $dateParts = explode(' ', $date);
        $datePart = explode('-', $dateParts[0]);
        $timePart = explode(':', $dateParts[1]);
        return new \Zend_Json_Expr(sprintf('new Date(%d, %d, %d, %d, %d, %d)', $datePart[0], $datePart[1] - 1, $datePart[2],
            $timePart[0], $timePart[1], $timePart[2]
        ));
    }

    /**
     * @param string $date
     * @return string
     */
    public function convertDateToLocale($date)
    {
        return $date = $this->localeDate
            ->date($date)
            ->format(DateTime::DATETIME_PHP_FORMAT);
    }

    /**
     * @return mixed
     */
    public function getWebTransactionCollection()
    {
        /** @var \Potato\PerformanceMonitoring\Model\ResourceModel\TransactionWeb\Collection $collection */
        $collection = $this->transactionWebCollectionFactory->create();
        return $collection
            ->setDateFilter(
                $this->getDateFrom(),
                $this->getDateTo()
            )
            ->setLimit($this->config->getWebTransactionsLimit());
    }

    /**
     * @return bool
     */
    public function canGroup()
    {
        if (strtotime($this->getDateTo()) -  strtotime($this->getDateFrom()) > self::TIME_LIMIT) {
            return true;
        }
        return false;
    }

    /**
     * System transactions chart data
     *
     * @param string $fieldName
     * @return mixed
     */
    public function getSystemTransactionsChartData($fieldName)
    {
        $groupedDataFlag = false;
        if ($this->canGroup()) {
            $groupedDataFlag = true;
        }

        /** @var \Potato\PerformanceMonitoring\Model\ResourceModel\TransactionSystem\Collection $collection */
        $collection = $this->transactionSystemCollectionFactory->create();
        $collection
            ->setDateFilter(
                $this->getDateFrom(),
                $this->getDateTo()
            );
        if ($unusedApplications = $this->config->getUnusedApplications()) {
            $collection->setAppFilter($unusedApplications);
        }
        $collection->orderByDate();
        
        if ($groupedDataFlag) {
            $collection->groupByHour($fieldName);
        }
        //prepare chart data
        $data = [
            ['DateTime']
        ];
        foreach ($collection as $row) {
            if ($groupedDataFlag) {
                $data[0][md5('(Max) ' . $row->getCmd())] = '(Max) ' . $row->getCmd();
                $data[0][md5('(Avg) ' . $row->getCmd())] = '(Avg) ' . $row->getCmd();
                continue;
            }
            $data[0][md5($row->getCmd())] = $row->getCmd();
            $data[0][md5($row->getCmd() . 'tooltip')] = [
                'type' => 'string',
                'role' => 'tooltip',
                'p' => ['html' => true]
            ];
        }

        foreach ($collection as $row) {
            $hash = md5($row->getCreatedAt());
            if (!array_key_exists($hash, $data)) {
                foreach ($data[0] as $key => $head) {
                    $data[$hash][$key] = 0;
                }
            }
            $data[$hash][0] = $this->formatDateToJs($row->getCreatedAt());
            if ($groupedDataFlag) {
                $data[$hash][md5('(Max) ' . $row->getCmd())] = (float)$row->getMax();
                $data[$hash][md5('(Avg) ' . $row->getCmd())] = (float)$row->getAvg();
                continue;
            }
            $data[$hash][md5($row->getCmd() . 'tooltip')] = $this->getTooltip($row, $fieldName);
            $data[$hash][md5($row->getCmd())] = (float)$row->getData($fieldName);
        }

        $newData = [];
        foreach ($data as $line) {
            $newData[] = array_values($line);
        }
        $newData[0][0] = [
            'label' => __('DateTime'),
            'type' => 'datetime'
        ];

        $zendJson = new \Zend_Json();
        return $zendJson::encode($newData, false, array('enableJsonExprFinder' => true));
    }

    /**
     * Tooltip for system transactions chart
     *
     * @param bool $locale
     * @param \Magento\Framework\DataObject $row
     * @param string $fieldName
     * @return string
     */
    protected function getTooltip(\Magento\Framework\DataObject $row, $fieldName, $locale=false)
    {
        $date = $row->getCreatedAt();
        if ($locale) {
            $date = $this->convertDateToLocale($date);
        }
        $result = "<div class='system-tooltip'><p><b>"
            . date('F j, Y, h:i:s A', strtotime($date)) . "</b></p>"
        ;
        $result .= "<p>" . $row->getCmd() . ": <b>" . (float)$row->getData($fieldName) ."</b></p>";
        $processes = $row->getProcesses();
        if (is_string($processes)) {
            if ($this->serializer) {
                $processes = $this->serializer->unserialize($processes);
            } else {
                $processes = unserialize($processes);
            }
            
        }
        if (empty($processes)) {
            return $result . '</div>';
        }
        $result .= "<table class='table table-striped'><thead><tr><th>" . __("pid")
            . "</th><th>" . __($fieldName) . "</th><th>" . __("cmd") . "</th></tr></thead><tbody>"
        ;
        foreach ($processes as $process) {
            $result .= "<tr><td>" . $process['pid'] . "</td><td>" . (float)$process[$fieldName] . "</td><td>"
                . $process['cmd'] . "</td></tr>"
            ;
        }
        $result .= "</tbody></table></div>";
        return $result;
    }
}