<?php
namespace Potato\PerformanceMonitoring\Model\ResourceModel;

use Magento\Framework\Api;
use Potato\PerformanceMonitoring\Api as PerformanceMonitoringApi;
use Potato\PerformanceMonitoring\Model as PerformanceMonitoringModel;
use Magento\Framework\Api\SearchCriteriaBuilder;

/**
 * Class MemoryRepository
 */
class MemoryRepository implements PerformanceMonitoringApi\MemoryRepositoryInterface
{
    /** @var PerformanceMonitoringModel\MemoryFactory  */
    protected $memoryFactory;

    /** @var PerformanceMonitoringModel\MemoryRegistry  */
    protected $memoryRegistry;

    /** @var PerformanceMonitoringApi\Data\MemorySearchResultsInterfaceFactory  */
    protected $searchResultsFactory;

    /** @var Api\ExtensibleDataObjectConverter  */
    protected $extensibleDataObjectConverter;

    /** @var Memory  */
    protected $memoryResource;

    /** @var SearchCriteriaBuilder  */
    protected $searchBuilder;

    /**
     * MemoryRepository constructor.
     * @param PerformanceMonitoringModel\MemoryFactory $memoryFactory
     * @param PerformanceMonitoringModel\MemoryRegistry $memoryRegistry
     * @param PerformanceMonitoringApi\Data\MemorySearchResultsInterfaceFactory $searchResultsFactory
     * @param Api\ExtensibleDataObjectConverter $extensibleDataObjectConverter
     * @param Memory $memoryResource
     * @param SearchCriteriaBuilder $searchBuilder
     */
    public function __construct(
        PerformanceMonitoringModel\MemoryFactory $memoryFactory,
        PerformanceMonitoringModel\MemoryRegistry $memoryRegistry,
        PerformanceMonitoringApi\Data\MemorySearchResultsInterfaceFactory $searchResultsFactory,
        Api\ExtensibleDataObjectConverter $extensibleDataObjectConverter,
        Memory $memoryResource,
        SearchCriteriaBuilder $searchBuilder
    ) {
        $this->memoryFactory = $memoryFactory;
        $this->memoryRegistry = $memoryRegistry;
        $this->memoryResource = $memoryResource;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->extensibleDataObjectConverter = $extensibleDataObjectConverter;
        $this->searchBuilder = $searchBuilder;
    }

    /**
     * @param \Potato\PerformanceMonitoring\Api\Data\MemoryInterface $memory
     * @return Memory
     */
    public function save(PerformanceMonitoringApi\Data\MemoryInterface $memory)
    {
        $memoryData = $this->extensibleDataObjectConverter->toNestedArray(
            $memory,
            [],
            PerformanceMonitoringApi\Data\MemoryInterface::class
        );
        $memoryModel = $this->memoryFactory->create();
        $memoryModel->addData($memoryData);
        $memoryModel->setId($memory->getId());
        $this->memoryResource->save($memoryModel);
        $this->memoryRegistry->push($memoryModel);
        $savedObject = $this->get($memoryModel->getId());
        return $savedObject;
    }

    /**
     * @param int $memoryId
     * @return PerformanceMonitoringApi\Data\MemoryInterface
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function get($memoryId)
    {
        $memoryModel = $this->memoryRegistry->retrieve($memoryId);
        return $memoryModel->getDataModel();
    }

    /**
     * @param PerformanceMonitoringApi\Data\MemoryInterface $memory
     * @return bool
     */
    public function delete(PerformanceMonitoringApi\Data\MemoryInterface $memory)
    {
        return $this->deleteById($memory->getId());
    }

    /**
     * @param int $memoryId
     * @return bool
     * @throws \Exception
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function deleteById($memoryId)
    {
        $memoryModel = $this->memoryRegistry->retrieve($memoryId);
        $memoryModel->getResource()->delete($memoryModel);
        $this->memoryRegistry->remove($memoryId);
        return true;
    }
    
    /**
     * @param Api\SearchCriteriaInterface $searchCriteria
     * @return mixed
     */
    public function getList(Api\SearchCriteriaInterface $searchCriteria)
    {
        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($searchCriteria);
        $collection = $this->memoryFactory->create()->getCollection();
        foreach ($searchCriteria->getFilterGroups() as $filterGroup) {
            $fields = [];
            $conditions = [];
            foreach ($filterGroup->getFilters() as $filter) {
                $condition = $filter->getConditionType() ? $filter->getConditionType() : 'eq';
                $fields[] = $filter->getField();
                $conditions[] = [$condition => $filter->getValue()];

            }
            if ($fields) {
                $collection->addFieldToFilter($fields, $conditions);
            }
        }
        $searchResults->setTotalCount($collection->getSize());
        $sortOrders = $searchCriteria->getSortOrders();
        if ($sortOrders) {
            /** @var Api\SortOrder $sortOrder */
            foreach ($searchCriteria->getSortOrders() as $sortOrder) {
                $collection->addOrder(
                    $sortOrder->getField(),
                    ($sortOrder->getDirection() == Api\SortOrder::SORT_ASC) ? 'ASC' : 'DESC'
                );
            }
        }
        $collection->setCurPage($searchCriteria->getCurrentPage());
        $collection->setPageSize($searchCriteria->getPageSize());

        $memorys = [];
        foreach ($collection as $memoryModel) {
            $memorys[] = $memoryModel->getDataModel();
        }
        $searchResults->setItems($memorys);
        return $searchResults;
    }

    /**
     * @param string $date
     * @return $this
     */
    public function removeDataByDate($date)
    {
        $criteria = $this
            ->searchBuilder
            ->addFilter(
                'created_at',
                $date,
                'lteq'
            )
            ->create();
        $items = $this->getList($criteria)->getItems();
        foreach ($items as $item) {
            $this->delete($item);
        }
        return $this;
    }

    /**
     * Create new empty memory model
     * @return PerformanceMonitoringApi\Data\MemoryInterface
     */
    public function create()
    {
        return $this->memoryRegistry->create();
    }
}
